/************************************************************************
 *   IRC - Internet Relay Chat, include/modules.h
 *   Copyright (C) 1990 Jarkko Oikarinen
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: modules.h,v 1.3 2001/09/16 22:45:05 asuffield Exp $
 */

#ifndef INCLUDED_modules_h
#define INCLUDED_modules_h
#include "config.h"
#include "setup.h"
#include "parse.h"
#ifdef HAVE_SYS_PARAM_H
#include <sys/param.h>
#endif

#include "ircd_handler.h"
#include "msg.h"
#include "memory.h"

typedef struct module_handle *mod_init_func(void);
typedef void mod_deinit_func(int);

#define UNLOAD_USER 0
#define UNLOAD_RELOAD 1
#define UNLOAD_SHUTDOWN 2
#define UNLOAD_ERROR 3

struct message_link
{
  struct Message *msg;
  struct message_link *next;
};

struct module_handle
{
  char *name;
  char *desc;
  char *version;
  struct message_link *msgs;
  void *code;		/* Where the actual code is loaded, NULL if in the core binary */
  mod_deinit_func *deinit;
};

#ifndef STATIC_MODULES
extern mod_init_func _modinit;
extern mod_deinit_func _moddeinit;

/* add a path */
void mod_add_path(char *path);
void mod_clear_paths(void);

extern int probe_module(char *);
extern int load_module(char *);
extern char *irc_basename(char *);
extern void load_module_conf(char *);
extern void clear_module_conf(void);
#endif /* STATIC_MODULES */

extern int unload_module(struct module_handle *, int type);
extern struct module_handle *findmodule_byname(char *);

extern void modules_init(void);
extern void modules_deinit(void);

extern struct module_handle *register_module(const char *, const char *, const char *, mod_deinit_func *);
extern void deregister_module(struct module_handle *, int);
extern void free_module_handle(struct module_handle *);

#ifdef STATIC_MODULES
extern void call_module_inits(void);
#endif

#ifdef STATIC_MODULES
#define MODULE(X) static char *_name = X;
#else
#define MODULE(X) char *_dancer_version = VERSION; char *_name = X;
#endif

#endif /* INCLUDED_modules_h */
